
/*
 * Piezo Energy Plant
 * ==================
 * 
 * 2022 Markus Opitz
 * www.instructables.com
 */


// Connect D0 + RST to wake up from deep sleep

// WiFi, Thingspeak, save value
#include <ESP8266WiFi.h>
const char* ssid = "*****";   // your WiFi
const char* pass = "*****";   // your password

#include "ThingSpeak.h"
#include <RTCVars.h>
RTCVars state; // create the state object
#include "FS.h"

WiFiClient client;

// Thingspeak.com
#define SECRET_CH_ID *******                    // your channel number
#define SECRET_WRITE_APIKEY "****************"  // your API key
unsigned long myChannelNumber = SECRET_CH_ID;
const char * myWriteAPIKey = SECRET_WRITE_APIKEY;


// Data
#include <Wire.h>
#include <Adafruit_INA219.h>
Adafruit_INA219 ina219;

float shuntvoltage = 0;
float busvoltage = 0;
float loadvoltage = 0;
float current_mA = 0;
float power_mW = 0;
unsigned long previousMillis = 0;
unsigned long interval = 100;
float energy = 0;
float capacity=0;

// ??
String myStatus = "";
//int zahl;
int counter;    // Wert


// *************** connect RST to D0  !   ****************


void setup() {
  Serial.begin(115200);  // Initialize serial
  delay(10);
  pinMode(D7, OUTPUT);
  
  WiFi.mode(WIFI_STA); 
  ThingSpeak.begin(client);  // Initialize ThingSpeak
  state.registerVar( &counter );    // send a pointer to variable


 //sensor
   ina219.begin();
   ina219.setCalibration_32V_1A(); 
}



void loop() {

  // = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
  // get new sensor data
  // = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

  read_sensor_data();
  //display_data();
  delay(20);

// Connect or reconnect to WiFi - - - - - - - - - - - - - - - - - 
  int WifiCounter = 0;

  digitalWrite(D7, HIGH); // LED on
  Serial.print("Connecting to "); Serial.println(ssid);
  WiFi.begin(ssid, pass);
    while(WiFi.status() != WL_CONNECTED){
      //WiFi.begin(ssid, pass);
      Serial.print(".");
      if (WifiCounter > 12){
        Serial.println("No connection. Sleep 180 secs");
        ESP.deepSleep(180e6); // sleep 180 seconds and restart
        ESP.restart();
      }
      WifiCounter++;
      delay(1000);     
    }
    Serial.println("\nConnected.");
         Serial.println("");
         Serial.print("Connected: ");
         Serial.println(WiFi.SSID());
  //}


  ThingSpeak.setField(6, busvoltage);  // set the fields with the values
  ThingSpeak.setStatus(myStatus);  // set the status
  
  // Write to ThingSpeak. 
  int x = ThingSpeak.writeFields(myChannelNumber, myWriteAPIKey);
  if(x == 200){
    Serial.println("Channel update successful.");
  }
  else{
    Serial.println("Problem updating channel. HTTP error code " + String(x));
  }
 

  WiFi.disconnect();
  delay(2000);
  digitalWrite(D7, LOW);
  
  Serial.println("\nSleep again 180 sec");
  //ESP.deepSleep(10e6); // sleep 10 seconds and restart
  //ESP.deepSleep(60e6); // sleep 60 seconds and restart
  ESP.deepSleep(180e6); // sleep 3 min and restart
  //ESP.deepSleep(1800e6); // sleep 1800 seconds and restart (= 30 min)
  //ESP.deepSleep(3600e6); // sleep 3600 seconds and restart (=1 hour)
  //ESP.deepSleep(3680e6);  // sleep 3680 seconds and restart (=1 real hour)
  ESP.restart(); 
}

void read_sensor_data(){  
  shuntvoltage = ina219.getShuntVoltage_mV();
  busvoltage = ina219.getBusVoltage_V();
    if (busvoltage > 6){
    busvoltage = 6;
  }
  current_mA = ina219.getCurrent_mA();
  if( current_mA <0)
  {
    current_mA=0;
  }

  loadvoltage = busvoltage + (shuntvoltage / 1000);
  power_mW = loadvoltage*current_mA;  
  capacity = current_mA / 3600;
  energy = energy + loadvoltage * current_mA / 3600;
}

void display_data(){
  // show data 
 Serial.print("Bus Voltage:   "); Serial.print(busvoltage); Serial.println(" V");
 Serial.print("Current:       "); Serial.print(current_mA); Serial.println(" mA");
 Serial.print("Power:         "); Serial.print(power_mW); Serial.println(" mW");
 Serial.print("Energy:        "); Serial.print(energy, 3); Serial.println(" mWh");
 Serial.print("Capacity:      "); Serial.print(capacity, 3); Serial.println(" mAh");
 
Serial.println(" ");Serial.println(" ");

  }
